package com.jiayue.ssi.config.security;

import com.jiayue.ssi.constant.CacheConstants;
import com.jiayue.ssi.util.LocalCache;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.annotation.Order;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.GenericFilterBean;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
* 邮箱口令过滤器
*
* @author xsl
* @since 2023/02/20
*/
@RequiredArgsConstructor
@Order(2)
public class MailCodeFilter extends GenericFilterBean {
    private String defaultFilterProcessUrl = "/user/login";

    @Override
    public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain)
            throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) req;
        HttpServletResponse response = (HttpServletResponse) res;
        if ("POST".equalsIgnoreCase(request.getMethod()) && defaultFilterProcessUrl.equals(request.getServletPath())) {
            // 验证码验证
            String username = request.getParameter("username");
            Object mailCode = LocalCache.get(CacheConstants.MAIL_CODE_KEY + username);

            // 校验服务端验证码
            if (mailCode==null || "".equals(mailCode)){
                response.addHeader("Access-Control-Allow-Origin", "*");
                response.setContentType("text/html;charset=UTF-8");
                response.setStatus(401);
                response.getWriter().write("邮箱口令无效！");
                return;
            }
            // 页面录入的邮箱口令
            String mailbox = request.getParameter("mailbox");

            // 校验页面验证码
            if (StringUtils.isEmpty(mailbox)) {
                response.addHeader("Access-Control-Allow-Origin", "*");
                response.setContentType("text/html;charset=UTF-8");
                response.setStatus(401);
                response.getWriter().write("非法访问,邮箱口令错误！");
                return;
            }
            if (mailbox.length()!=6){
                response.addHeader("Access-Control-Allow-Origin", "*");
                response.setContentType("text/html;charset=UTF-8");
                response.setStatus(401);
                response.getWriter().write("需要6位邮箱口令！");
                return;
            }
            if (!String.valueOf(mailCode).toLowerCase().equals(mailbox.toLowerCase())) {
                response.addHeader("Access-Control-Allow-Origin", "*");
                response.setContentType("text/html;charset=UTF-8");
                response.setStatus(401);
                response.getWriter().write("邮箱口令错误！");
                return;
            }
        }
        chain.doFilter(request, response);
    }
}
