package com.jiayue.ssi.config.security;

import com.jiayue.ssi.constant.CacheConstants;
import com.jiayue.ssi.util.LocalCache;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.annotation.Order;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.GenericFilterBean;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
* 验证码过滤器
*
* @author xsl
* @since 2023/02/20
*/
@RequiredArgsConstructor
@Order(1)
public class VerifyCodeFilter extends GenericFilterBean {
    private String defaultFilterProcessUrl = "/user/login";

    @Override
    public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain)
            throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) req;
        HttpServletResponse response = (HttpServletResponse) res;
        if ("POST".equalsIgnoreCase(request.getMethod()) && defaultFilterProcessUrl.equals(request.getServletPath())) {
            // 验证码验证
            String requestCaptcha = request.getParameter("code");
            String verifyuuid = request.getParameter("verifyuuid");
            Object uuidObj = LocalCache.get(CacheConstants.CAPTCHA_CODE_KEY + verifyuuid);
            // 校验服务端验证码
            if (uuidObj==null || "".equals(uuidObj)){
                response.addHeader("Access-Control-Allow-Origin", "*");
                response.setContentType("text/html;charset=UTF-8");
                response.setStatus(401);
                response.getWriter().write("验证码无效！");
                return;
            }
            // 校验页面验证码
            if (StringUtils.isEmpty(requestCaptcha)) {
                response.addHeader("Access-Control-Allow-Origin", "*");
                response.setContentType("text/html;charset=UTF-8");
                response.setStatus(401);
                response.getWriter().write("非法访问,验证码错误！");
                return;
            }
            if (requestCaptcha.length()!=4){
                response.addHeader("Access-Control-Allow-Origin", "*");
                response.setContentType("text/html;charset=UTF-8");
                response.setStatus(401);
                response.getWriter().write("需要4位验证码！");
                return;
            }
            if (!String.valueOf(uuidObj).toLowerCase().equals(requestCaptcha.toLowerCase())) {
                response.addHeader("Access-Control-Allow-Origin", "*");
                response.setContentType("text/html;charset=UTF-8");
                response.setStatus(401);
                response.getWriter().write("验证码错误！");
                return;
            }
        }
        chain.doFilter(request, response);
    }
}
